;;;
;;;  CHGATT.LSP   Version 1.0
;;;
;;;  ALSO REQUIRES CHGATT.DCL
;;;
;;;  Copyright (C) 1996 by Jay Garnett

;;;
;;;  Permission to use, copy, modify, and distribute this software
;;;  for any purpose and without fee is hereby granted, provided
;;;  that the above copyright notice appears in all copies and
;;;  that both that copyright notice and the limited warranty 
;;;  below appear in all supporting documentation.
;;;
;;;  JAY GARNETT PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS.
;;;  JAY GARNETT SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
;;;  MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE. JAY GARNETT
;;;  DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
;;;  UNINTERRUPTED OR ERROR FREE.
;;;
;;;  DESCRIPTION
;;;
;;;  Allows user to change the layer or text style of an attribute via a dialog box.
;;;
;;;  By Jay Garnett
;;;  Bolingbrook, IL
;;;  
;;;  E-Mail jgarnett@enteract.com
;;;
;;;Returns a sorted list of all names (text style, layers, ect.) in specified table.
;;;
(defun Get_Tbl_Names( TABLE / DATA TBL_DATA)
   (while (setq TBL_DATA(tblnext TABLE (null DATA)))
      (if (not DATA)
         (setq DATA(list(cdr(assoc 2 TBL_DATA))))
         (setq DATA(append DATA(list(cdr(assoc 2 TBL_DATA)))))
      )
   )
   (acad_strlsort DATA)
)

(defun c:CHGATT( / ATT ATT_DATA BRANCH DCL_ID LAY LAY_ID LOOP LYRS STYLS STY_ID STRNG STY)
   (setq DCL_ID (load_dialog "CHGATT.DCL"))                        ;load DCL file
   (if (< DCL_ID 0)(progn(alert "Can't find\nCHGATT.DCL")(exit)))  ;Exit if file not found
   
   (while (not ATT)
      (setq ATT(car(nentsel)))
      (setq ATT_DATA(entget ATT))
      (if (/=(cdr(assoc 0 ATT_DATA))"ATTRIB")
         (progn
            (setq ATT nil)
            (prompt "\nEntity selected was not an Attribute")
         )
      )
   )
   (setq STRNG(cdr(assoc 1 ATT_DATA))
      STY(cdr(assoc 7 ATT_DATA))
      LAY(cdr(assoc 8 ATT_DATA))
      STRNG(strcat "Value: " STRNG)
      STYLS(Get_Tbl_Names "STYLE")
      LYRS(Get_Tbl_Names "LAYER")
   )
   
   (setq LOOP 0)
   (while (not LAY_ID)
      (if (= LAY (nth LOOP LYRS))(setq LAY_ID (itoa LOOP)))
      (setq LOOP (1+ LOOP))
   )
   (setq LOOP 0)
   (while(not STY_ID)
      (if (= STY (nth LOOP STYLS))(setq STY_ID (itoa LOOP)))
      (setq LOOP (1+ LOOP))
   )
   
   (if (not (new_dialog "CHGATT" DCL_ID))(exit))
   
   (start_list "$STY") (mapcar 'add_list STYLS) (end_list)
   (start_list "$LAY") (mapcar 'add_list LYRS)  (end_list)

   (action_tile "$LAY" "(setq NEW_LAY $value)")
   (action_tile "$STY" "(setq NEW_STY $value)")

   (set_tile "$TVAL" STRNG)
   (set_tile "$LAY" LAY_ID)
   (set_tile "$STY" STY_ID)

   (setq BRANCH (start_dialog))
   (if (= BRANCH 1)
      (progn
         (if NEW_STY(setq ATT_DATA(subst (cons 7 (nth (atoi NEW_STY) STYLS))
                                                    (assoc 7 ATT_DATA) ATT_DATA)))
         (if NEW_LAY(setq ATT_DATA(subst (cons 8 (nth (atoi NEW_LAY) LYRS))
                                                    (assoc 8 ATT_DATA) ATT_DATA)))
         (entmod ATT_DATA)
         (entupd(cdar ATT_DATA))
      )
      
   )
   (unload_dialog DCL_ID)
)
(prompt "\n* CHGATT.LSP Copyright (c) 1996, Jay Garnett")
(prompt "\n* CHGATT.LSP loaded. Type CHGATT to run.")
(princ)
;;;------------------------------------------------------------------